@@ -416,10 +416,10 @@ static void collection_realize(GtkWidget *widget)
 
 	gdk_window_set_user_data(widget->window, widget);
 	gdk_window_set_background(widget->window,
-			&widget->style->bg[GTK_STATE_NORMAL]);
+			&widget->style->base[GTK_STATE_NORMAL]);
 
-	bg = &widget->style->bg[GTK_STATE_NORMAL];
-	fg = &widget->style->fg[GTK_STATE_NORMAL];
+	bg = &widget->style->base[GTK_STATE_NORMAL];
+	fg = &widget->style->text[GTK_STATE_NORMAL];
 	xor_values.function = GDK_XOR;
 	xor_values.foreground.pixel = fg->pixel ^ bg->pixel;
 	collection->xor_gc = gdk_gc_new_with_values(widget->window,
@@ -544,10 +544,13 @@ static gint collection_expose(GtkWidget *widget, GdkEventExpose *event)
 	/* Note about 'detail' argument:
 	 * - If set to "base", lighthouse theme will crash
 	 * - If set to NULL, cleanice theme will crash
+	 *
+	 * Clear the background only if we have a background pixmap.
 	 */
-	gtk_paint_flat_box(widget->style, widget->window, GTK_STATE_NORMAL, 
-		      GTK_SHADOW_NONE, &event->area,
-		      widget, "collection", 0, 0, -1, -1);
+	if (widget->style->bg_pixmap[GTK_STATE_NORMAL])
+		gtk_paint_flat_box(widget->style, widget->window, GTK_STATE_NORMAL, 
+				   GTK_SHADOW_NONE, &event->area,
+				   widget, "collection", 0, 0, -1, -1);
 
 	collection = COLLECTION(widget);
 
@@ -1758,20 +1761,28 @@ void collection_move_cursor(Collection *collection, int drow, int dcol)
 			row = last;
 		else {
 			row += drow;
-			if (!collection->vertical_order) 
-		{
+			if (collection->vertical_order) 
+			{
+				if (row >= total_rows)
+				{
+					row = 0;
+					col += 1;
+				}
+			}
+			else 
+			{
 				row = MAX(row, 0); 
 				row = MIN(row, total_rows - 1);
+			}
 		}
 	}
-	}
 
 	item = collection_rowcol_to_item(collection, row, col);
 
 	item = MAX(item, 0);
 	item = MIN(item, collection->number_of_items-1);
 
-		collection_set_cursor_item(collection, item, TRUE);
+	collection_set_cursor_item(collection, item, TRUE);
 }
 
 /* Start a lasso box drag */
