@@ -1,8 +1,6 @@
 /*
- * $Id: display.c,v 1.132 2005/07/29 22:06:46 kerofin Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -57,6 +55,7 @@
 #include "diritem.h"
 #include "fscache.h"
 #include "view_iface.h"
+#include "xtypes.h"
 
 #define HUGE_WRAP (1.5 * o_large_width.int_value)
 
@@ -75,6 +74,7 @@ Option o_display_show_full_type;
 Option o_display_inherit_options;
 static Option o_filer_change_size_num;
 Option o_vertical_order_small, o_vertical_order_large;
+Option o_xattr_show;
 
 /* Static prototypes */
 static void display_details_set(FilerWindow *filer_window, DetailsType details);
@@ -105,14 +105,37 @@ void display_init()
 	option_add_int(&o_filer_change_size_num, "filer_change_size_num", 30); 
 	option_add_int(&o_vertical_order_small, "vertical_order_small", FALSE);
 	option_add_int(&o_vertical_order_large, "vertical_order_large", FALSE);
+	option_add_int(&o_xattr_show, "xattr_show", TRUE);
 
 	option_add_notify(options_changed);
 }
 
-static void draw_emblem_on_icon(GdkWindow *window, MaskedPixmap *image,
+void draw_emblem_on_icon(GdkWindow *window, GtkStyle   *style,
+				const char *stock_id,
 				int *x, int y)
 {
-	gdk_pixbuf_render_to_drawable_alpha(image->pixbuf,
+	GtkIconSet *icon_set;
+	GdkPixbuf  *pixbuf;
+
+	icon_set = gtk_style_lookup_icon_set(style,
+					     stock_id);
+	if (icon_set)
+	{
+		pixbuf = gtk_icon_set_render_icon(icon_set,
+						  style,
+						  GTK_TEXT_DIR_LTR,
+						  GTK_STATE_NORMAL,
+						  mount_icon_size,
+						  NULL,
+						  NULL);
+	}
+	else
+	{
+		pixbuf=im_unknown->pixbuf;
+		g_object_ref(pixbuf);
+	}
+	
+	gdk_pixbuf_render_to_drawable_alpha(pixbuf,
 				window,
 				0, 0, 				/* src */
 				*x, y,		                /* dest */
@@ -120,18 +143,21 @@ static void draw_emblem_on_icon(GdkWindow *window, MaskedPixmap *image,
 				GDK_PIXBUF_ALPHA_FULL, 128,	/* (unused) */
 				GDK_RGB_DITHER_NORMAL, 0, 0);
 	
-	*x+=image->width+1;
+	*x+=gdk_pixbuf_get_width(pixbuf)+1;
+	g_object_unref(pixbuf);
 }
 
 /* Draw this icon (including any symlink or mount symbol) inside the
  * given rectangle.
  */
-void draw_huge_icon(GdkWindow *window, GdkRectangle *area,
-		   DirItem  *item, MaskedPixmap *image, gboolean selected)
+void draw_huge_icon(GdkWindow *window, GtkStyle *style, GdkRectangle *area,
+		   DirItem  *item, MaskedPixmap *image, gboolean selected,
+		   GdkColor *color)
 {
 	int		width, height;
 	int		image_x;
 	int		image_y;
+	GdkPixbuf	*pixbuf;
 
 	if (!image)
 		return;
@@ -141,9 +167,12 @@ void draw_huge_icon(GdkWindow *window, GdkRectangle *area,
 	image_x = area->x + ((area->width - width) >> 1);
 	image_y = MAX(0, area->height - height - 6);
 
+	pixbuf = selected
+		? create_spotlight_pixbuf(image->huge_pixbuf, color)
+		: image->huge_pixbuf;
+
 	gdk_pixbuf_render_to_drawable_alpha(
-			selected ? image->huge_pixbuf_lit
-				 : image->huge_pixbuf,
+			pixbuf,
 			window,
 			0, 0, 				/* src */
 			image_x, area->y + image_y,	/* dest */
@@ -151,20 +180,25 @@ void draw_huge_icon(GdkWindow *window, GdkRectangle *area,
 			GDK_PIXBUF_ALPHA_FULL, 128,	/* (unused) */
 			GDK_RGB_DITHER_NORMAL, 0, 0);
 
+	if (selected)
+		g_object_unref(pixbuf);
+
 	if (item->flags & ITEM_FLAG_MOUNT_POINT)
 	{
-		MaskedPixmap	*mp = item->flags & ITEM_FLAG_MOUNTED
-					? im_mounted
-					: im_unmounted;
-		draw_emblem_on_icon(window, mp, &image_x, area->y + 2);
+		const char *mp = item->flags & ITEM_FLAG_MOUNTED
+					? ROX_STOCK_MOUNTED
+					: ROX_STOCK_MOUNT;
+		draw_emblem_on_icon(window, style, mp, &image_x, area->y + 2);
 	}
 	if (item->flags & ITEM_FLAG_SYMLINK)
 	{
-		draw_emblem_on_icon(window, im_symlink, &image_x, area->y + 2);
+		draw_emblem_on_icon(window, style, ROX_STOCK_SYMLINK,
+				    &image_x, area->y + 2);
 	}
-	if (item->flags & ITEM_FLAG_HAS_XATTR)
+	if ((item->flags & ITEM_FLAG_HAS_XATTR) && o_xattr_show.int_value)
 	{
-		draw_emblem_on_icon(window, im_xattr, &image_x, area->y + 2);
+		draw_emblem_on_icon(window, style, ROX_STOCK_XATTR,
+				    &image_x, area->y + 2);
 	}
 }
 
@@ -172,15 +206,18 @@ void draw_huge_icon(GdkWindow *window, GdkRectangle *area,
  * given rectangle.
  */
 void draw_large_icon(GdkWindow *window,
+		     GtkStyle *style,
 		     GdkRectangle *area,
 		     DirItem  *item,
 		     MaskedPixmap *image,
-		     gboolean selected)
+		     gboolean selected,
+		     GdkColor *color)
 {
 	int	width;
 	int	height;
 	int	image_x;
 	int	image_y;
+	GdkPixbuf *pixbuf;
 
 	if (!image)
 		return;
@@ -190,8 +227,12 @@ void draw_large_icon(GdkWindow *window,
 	image_x = area->x + ((area->width - width) >> 1);
 	image_y = MAX(0, area->height - height - 6);
 
+	pixbuf = selected
+		? create_spotlight_pixbuf(image->pixbuf, color)
+		: image->pixbuf;
+
 	gdk_pixbuf_render_to_drawable_alpha(
-			selected ? image->pixbuf_lit : image->pixbuf,
+			pixbuf,
 			window,
 			0, 0, 				/* src */
 			image_x, area->y + image_y,	/* dest */
@@ -199,27 +240,34 @@ void draw_large_icon(GdkWindow *window,
 			GDK_PIXBUF_ALPHA_FULL, 128,	/* (unused) */
 			GDK_RGB_DITHER_NORMAL, 0, 0);
 
+	if (selected)
+		g_object_unref(pixbuf);
+
 	if (item->flags & ITEM_FLAG_MOUNT_POINT)
 	{
-		MaskedPixmap	*mp = item->flags & ITEM_FLAG_MOUNTED
-					? im_mounted
-					: im_unmounted;
-		draw_emblem_on_icon(window, mp, &image_x, area->y + 2);
+		const char *mp = item->flags & ITEM_FLAG_MOUNTED
+					? ROX_STOCK_MOUNTED
+					: ROX_STOCK_MOUNT;
+		draw_emblem_on_icon(window, style, mp, &image_x, area->y + 2);
 	}
 	if (item->flags & ITEM_FLAG_SYMLINK)
 	{
-		draw_emblem_on_icon(window, im_symlink, &image_x, area->y + 2);
+		draw_emblem_on_icon(window, style, ROX_STOCK_SYMLINK,
+				    &image_x, area->y + 2);
 	}
-	if (item->flags & ITEM_FLAG_HAS_XATTR)
+	if ((item->flags & ITEM_FLAG_HAS_XATTR) && o_xattr_show.int_value)
 	{
-		draw_emblem_on_icon(window, im_xattr, &image_x, area->y + 2);
+		draw_emblem_on_icon(window, style, ROX_STOCK_XATTR,
+				    &image_x, area->y + 2);
 	}
 }
 
-void draw_small_icon(GdkWindow *window, GdkRectangle *area,
-		     DirItem  *item, MaskedPixmap *image, gboolean selected)
+void draw_small_icon(GdkWindow *window, GtkStyle *style, GdkRectangle *area,
+		     DirItem  *item, MaskedPixmap *image, gboolean selected,
+		     GdkColor *color)
 {
 	int		width, height, image_x, image_y;
+	GdkPixbuf	*pixbuf;
 	
 	if (!image)
 		return;
@@ -232,8 +280,12 @@ void draw_small_icon(GdkWindow *window, GdkRectangle *area,
 	image_x = area->x + ((area->width - width) >> 1);
 	image_y = MAX(0, SMALL_HEIGHT - image->sm_height);
 		
+	pixbuf = selected
+		? create_spotlight_pixbuf(image->sm_pixbuf, color)
+		: image->sm_pixbuf;
+
 	gdk_pixbuf_render_to_drawable_alpha(
-			selected ? image->sm_pixbuf_lit : image->sm_pixbuf,
+			pixbuf,
 			window,
 			0, 0, 				/* src */
 			image_x, area->y + image_y,	/* dest */
@@ -241,20 +293,25 @@ void draw_small_icon(GdkWindow *window, GdkRectangle *area,
 			GDK_PIXBUF_ALPHA_FULL, 128,	/* (unused) */
 			GDK_RGB_DITHER_NORMAL, 0, 0);
 
+	if (selected)
+		g_object_unref(pixbuf);
+
 	if (item->flags & ITEM_FLAG_MOUNT_POINT)
 	{
-		MaskedPixmap	*mp = item->flags & ITEM_FLAG_MOUNTED
-					? im_mounted
-					: im_unmounted;
-		draw_emblem_on_icon(window, mp, &image_x, area->y + 2);
+		const char *mp = item->flags & ITEM_FLAG_MOUNTED
+					? ROX_STOCK_MOUNTED
+					: ROX_STOCK_MOUNT;
+		draw_emblem_on_icon(window, style, mp, &image_x, area->y + 2);
 	}
 	if (item->flags & ITEM_FLAG_SYMLINK)
 	{
-		draw_emblem_on_icon(window, im_symlink, &image_x, area->y + 8);
+		draw_emblem_on_icon(window, style, ROX_STOCK_SYMLINK,
+				    &image_x, area->y + 8);
 	}
-	if (item->flags & ITEM_FLAG_HAS_XATTR)
+	if ((item->flags & ITEM_FLAG_HAS_XATTR) && o_xattr_show.int_value)
 	{
-		draw_emblem_on_icon(window, im_xattr, &image_x, area->y + 8);
+		draw_emblem_on_icon(window, style, ROX_STOCK_XATTR,
+				    &image_x, area->y + 8);
 	}
 }
 
@@ -461,6 +518,20 @@ void display_set_hidden(FilerWindow *filer_window, gboolean hidden)
 	display_update_hidden(filer_window);
 }
 
+/* Set the 'Filter Directories' flag for this window */
+void display_set_filter_directories(FilerWindow *filer_window, gboolean filter_directories)
+{
+	if (filer_window->filter_directories == filter_directories)
+		return;
+
+	/*
+	filer_window->show_hidden = hidden;
+	*/
+	filer_set_filter_directories(filer_window, filter_directories);
+
+	display_update_hidden(filer_window);
+}
+
 void display_set_filter(FilerWindow *filer_window, FilterType type,
 			const gchar *filter_string)
 {
@@ -758,6 +829,7 @@ void display_update_view(FilerWindow *filer_window,
 	{
 		view->layout = gtk_widget_create_pango_layout(
 				filer_window->window, item->leafname);
+		pango_layout_set_auto_dir(view->layout, FALSE);
 	}
 	else
 	{
