@@ -104,8 +104,9 @@ typedef struct {
 	DetailsType	details_type;
 	DisplayStyle	display_style;
 
-	FilterType filter_type;
-	char *filter;
+	FilterType      filter_type;
+	char           *filter;
+	gboolean        filter_directories;
 } Settings;
 
 enum settings_flags{
@@ -119,6 +120,8 @@ enum settings_flags{
 	SET_FILTER=128,   /* filter_type, filter */
 };
 
+static GHashTable *unmount_prompt_actions = NULL;
+
 /* Static prototypes */
 static void attach(FilerWindow *filer_window);
 static void detach(FilerWindow *filer_window);
@@ -152,6 +155,8 @@ static gboolean drag_motion(GtkWidget		*widget,
                             guint		time,
 			    FilerWindow		*filer_window);
 
+static void load_learnt_mounts(void);
+static void save_learnt_mounts(void);
 static void load_settings(void);
 static void save_settings(void);
 static void check_settings(FilerWindow *filer_window);
@@ -170,6 +175,8 @@ static Option o_filer_view_type;
 Option o_filer_auto_resize, o_unique_filer_windows;
 Option o_filer_size_limit;
 
+#define ROX_RESPONSE_EJECT 99 /**< User clicked on Eject button */
+
 void filer_init(void)
 {
 	const gchar *ohost;
@@ -206,8 +213,8 @@ void filer_init(void)
 
 	if (dpyhost[0] && strcmp(ohost, dpyhost) != 0)
 	{
-	        /* Try the cannonical name for dpyhost (see our_host_name()
-	         * in support.c).
+		/* Try the cannonical name for dpyhost (see our_host_name()
+		 * in support.c).
 		 */
 	        struct hostent *ent;
 		
@@ -219,6 +226,7 @@ void filer_init(void)
 	g_free(dpyhost);
 
 	load_settings();
+	load_learnt_mounts();
 }
 
 static gboolean if_deleted(gpointer item, gpointer removed)
@@ -501,17 +509,39 @@ static char *get_ancestor_user_mount_point(const char *start)
 	}
 }
 
-static void umount_dialog_response(GtkWidget *dialog, int response, char *mount)
+static void unmount_dialog_response(GtkWidget *dialog,
+        int response, char *mount)
 {
-	if (response == GTK_RESPONSE_OK)
+	GList *list = NULL;
+	UnmountPrompt prompt_val = UNMOUNT_PROMPT_ASK;
+	
+	switch (response)
 	{
-		GList *list; 
-
+	case GTK_RESPONSE_OK:
 		list = g_list_prepend(NULL, mount);
 		action_mount(list, FALSE, FALSE, TRUE);
-		g_list_free(list);
-	}
+		prompt_val = UNMOUNT_PROMPT_UNMOUNT;
+		break;
 
+	case ROX_RESPONSE_EJECT:
+		list = g_list_prepend(NULL, mount);
+		action_eject(list);
+		prompt_val = UNMOUNT_PROMPT_EJECT;
+		break;
+
+	default:
+		prompt_val = UNMOUNT_PROMPT_NO_CHANGE;
+		break;
+	}
+	if (list)
+		g_list_free(list);
+
+	if (gtk_toggle_button_get_active(g_object_get_data(G_OBJECT(dialog),
+			"unmount_mem_btn")))
+	{
+	    filer_set_unmount_action(mount, prompt_val);
+	}
+					
 	g_free(mount);
 
 	gtk_widget_destroy(dialog);
@@ -527,7 +557,7 @@ static void umount_dialog_response(GtkWidget *dialog, int response, char *mount)
  */
 static void may_offer_unmount(FilerWindow *filer_window, char *mount)
 {
-	GtkWidget *dialog, *button;
+	GtkWidget *dialog, *button, *unmount_mem_btn;
 	GList	*next;
 	int len;
 	
@@ -556,12 +586,49 @@ static void may_offer_unmount(FilerWindow *filer_window, char *mount)
 		g_free(mount);
 		return;
 	}
+	
+	if (unmount_prompt_actions)
+	{
+		GList *list = NULL;
+		UnmountPrompt unmount_val = filer_get_unmount_action(mount);
+				
+		switch (unmount_val)
+		{
+		case UNMOUNT_PROMPT_UNMOUNT:
+			list = g_list_prepend(NULL, mount);
+			action_mount(list, FALSE, FALSE, TRUE);
+			break;
+		
+		case UNMOUNT_PROMPT_EJECT:
+			list = g_list_prepend(NULL, mount);
+			action_eject(list);
+			break;
+		
+		default:
+			break;
+		}
+		if (list)
+			g_list_free(list);
+		if (unmount_val != UNMOUNT_PROMPT_ASK)
+		{
+			g_free(mount);
+			return;
+		}
+	}
 
 	dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_QUESTION,
 			GTK_BUTTONS_NONE, 
 			_("Do you want to unmount this device?\n\n"
 			"Unmounting a device makes it safe to remove "
-			"the disk."), mount);
+			"the disk."));
+	
+	unmount_mem_btn = gtk_check_button_new_with_label(
+			_("Perform the same action in future for this mount point"));
+	gtk_widget_show(unmount_mem_btn);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), unmount_mem_btn,
+			FALSE, FALSE, 0);
+	g_object_set_data(G_OBJECT(dialog), "unmount_mem_btn",
+			unmount_mem_btn);
 
 	button = button_new_mixed(ROX_STOCK_MOUNTED, _("No change"));
 	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
@@ -575,8 +642,15 @@ static void may_offer_unmount(FilerWindow *filer_window, char *mount)
 					GTK_RESPONSE_OK);
 	gtk_widget_show(button);
 
+	/* We need a better icon, but I can't draw */
+	button = button_new_mixed(GTK_STOCK_UNDO, _("Eject"));
+	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+	gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button,
+					ROX_RESPONSE_EJECT);
+	gtk_widget_show(button);
+
 	g_signal_connect(G_OBJECT(dialog), "response",
-			G_CALLBACK(umount_dialog_response), mount);
+			G_CALLBACK(unmount_dialog_response), mount);
 
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog),
 			GTK_RESPONSE_OK);
@@ -1437,6 +1511,7 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 	filer_window->filter = FILER_SHOW_ALL;
 	filer_window->filter_string = NULL;
 	filer_window->regexp = NULL;
+	filer_window->filter_directories = FALSE;
 	
 	if (src_win && o_display_inherit_options.int_value)
 	{
@@ -1448,6 +1523,7 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 		filer_window->show_thumbs = src_win->show_thumbs;
 		filer_window->view_type = src_win->view_type;
 
+		filer_window->filter_directories = src_win->filter_directories;
 		filer_set_filter(filer_window, src_win->filter,
 				 src_win->filter_string);
 	}
@@ -1491,9 +1567,13 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 			filer_window->show_thumbs = dir_settings->show_thumbs;
 		
 		if (dir_settings->flags & SET_FILTER)
+		{
 			filer_set_filter(filer_window,
 					   dir_settings->filter_type,
 					   dir_settings->filter);
+			filer_set_filter_directories(filer_window,
+					     dir_settings->filter_directories);
+		}
 	}
 
 	/* Add all the user-interface elements & realise */
@@ -2630,7 +2710,13 @@ gint filer_motion_notify(FilerWindow *filer_window, GdkEventMotion *event)
 	
 	if (!view_get_selected(view, &iter))
 	{
-		if (event->state & GDK_BUTTON1_MASK)
+		/* If we drag an unselected item, select it only.
+		 * Unless we're also holding down Ctrl, in which case
+		 * it's probably unselected only because we
+		 * mis-interpreted the click as toggle-selected.
+		 */
+		if ((event->state & GDK_BUTTON1_MASK) &&
+		    !(event->state & GDK_CONTROL_MASK))
 		{
 			/* Select just this one */
 			filer_window->temp_item_selected = TRUE;
@@ -2922,7 +3008,9 @@ gboolean filer_match_filter(FilerWindow *filer_window, DirItem *item)
 	switch(filer_window->filter) {
 	case FILER_SHOW_GLOB:
 		return fnmatch(filer_window->filter_string,
-			       item->leafname, 0)==0;
+			       item->leafname, 0)==0 ||
+		  (item->base_type==TYPE_DIRECTORY &&
+		   !filer_window->filter_directories);
 		
 	case FILER_SHOW_ALL:
 	default:
@@ -2937,6 +3025,13 @@ void filer_set_hidden(FilerWindow *filer_window, gboolean hidden)
 	filer_window->show_hidden=hidden;
 }
 
+/* Provided to hide the implementation */
+void filer_set_filter_directories(FilerWindow *filer_window,
+        gboolean filter_directories)
+{
+	filer_window->filter_directories=filter_directories;
+}
+
 /* Set the filter type. Returns TRUE if the type has changed
  * (must call filer_detach_rescan).
  */
@@ -3069,12 +3164,93 @@ static void load_from_node(Settings *set, xmlDocPtr doc, xmlNodePtr node)
 	} else if(strcmp(node->name, "Filter") == 0) {
 		set->filter=g_strdup(str);
 		set->flags|=SET_FILTER;		
+	} else if(strcmp(node->name, "FilterDirectories") == 0) {
+		set->filter_directories=atoi(str);
+		set->flags|=SET_FILTER;		
 	}
 	
 	if(str)
 		xmlFree(str);
 }
 
+static void load_learnt_mounts(void)
+{
+	gchar *path;
+	gchar *buffer = NULL;
+	gsize len = 0;
+	gchar **entries;
+	int n;
+			
+	unmount_prompt_actions = g_hash_table_new_full(g_str_hash,
+			g_str_equal, g_free, NULL);
+
+	path = choices_find_xdg_path_load("Mounts", PROJECT, SITE);
+	if (!path)
+		return;
+	if (!g_file_get_contents(path, &buffer, &len, NULL))
+	{
+		g_free(path);
+		return;
+	}
+	g_free(path);
+	if (len)
+	{
+		buffer[len - 1] = 0;
+	}
+	else
+	{
+		g_free(buffer);
+		return;
+	}
+	
+	entries = g_strsplit(buffer, "\n", 0);
+	g_free(buffer);
+	for (n = 0; entries[n]; ++n)
+	{
+		gchar *p;
+		buffer = entries[n];
+		p = strrchr(buffer, ' ');
+		if (p && p > buffer) {
+			*p = 0;
+			g_hash_table_insert(unmount_prompt_actions, g_strdup(buffer),
+				GINT_TO_POINTER(atoi(p+1)));
+		}
+	}
+	g_strfreev(entries);
+}
+
+static void save_mount(char *path, gpointer value, FILE **pfp)
+{
+	int v;
+
+	if (!*pfp)
+	{
+		gchar *spath = choices_find_xdg_path_save("Mounts",
+				PROJECT, SITE, TRUE);
+		
+		if (!spath)
+			return;
+		*pfp = fopen(spath, "w");
+		g_free(spath);
+		if (!*pfp)
+			return;
+	}
+
+	if ((v = GPOINTER_TO_INT(value)) != UNMOUNT_PROMPT_ASK)
+		fprintf(*pfp, "%s %d\n", path, v);
+}
+
+static void save_learnt_mounts(void)
+{
+	FILE *fp = NULL;
+	
+	/* A GHashTableIter would be easier, but it's a relatively new feature */
+	if (unmount_prompt_actions)
+		g_hash_table_foreach(unmount_prompt_actions, (GHFunc) save_mount, &fp);
+	if (fp)
+		fclose(fp);
+}
+
 static void load_settings(void)
 {
 	gchar *path;
@@ -3186,7 +3362,9 @@ static void add_nodes(gpointer key, gpointer value, gpointer data)
 		xmlNewChild(sub, NULL, "FilterType", tmp);
 		g_free(tmp);
 		if(set->filter && set->filter[0])
-			xmlNewChild(sub, NULL, "Filter", set->filter);
+			xmlNewChild(sub, NULL, "Filter", set->filter);	
+		tmp=g_strdup_printf("%d", set->filter_directories);
+		xmlNewChild(sub, NULL, "FilterDirectories", tmp);
 	}
 }
 
@@ -3256,9 +3434,13 @@ static void check_settings(FilerWindow *filer_window)
 					   set->show_thumbs);
 		
 		if(set->flags & SET_FILTER)
+		{
 			display_set_filter(filer_window,
 					   set->filter_type,
 					   set->filter);
+			display_set_filter_directories(filer_window,
+					   set->filter_directories);
+		}
 	}
 
 }
@@ -3307,8 +3489,8 @@ static void settings_response(GtkWidget *window, gint response,
 void filer_save_settings(FilerWindow *fwin)
 {
 	SettingsWindow *set_win;
-	GtkWidget *vbox;
-	GtkWidget *path;
+	GtkWidget *vbox, *frame;
+	GtkWidget *path, *lbl;
 	gint x, y;
 	
 	Settings *set=settings_new(fwin->sym_path);
@@ -3344,6 +3526,7 @@ void filer_save_settings(FilerWindow *fwin)
 	set->filter_type=fwin->filter;
 	if(fwin->filter_string)
 		set->filter=g_strdup(fwin->filter_string);
+	set->filter_directories=fwin->filter_directories;
 
 	/* Store other parameters
 	*/
@@ -3367,9 +3550,20 @@ void filer_save_settings(FilerWindow *fwin)
 
 	vbox=GTK_DIALOG(set_win->window)->vbox;
 
+	lbl=gtk_label_new(_("<b>Save display settings for directory</b>"));
+	gtk_label_set_use_markup(GTK_LABEL(lbl), TRUE);
+	gtk_box_pack_start(GTK_BOX(vbox), lbl, FALSE, FALSE, 2);
+
 	path=gtk_label_new(set->path);
 	gtk_box_pack_start(GTK_BOX(vbox), path, FALSE, FALSE, 2);
 
+	frame=gtk_frame_new(_("Select settings to save"));
+	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 2);
+
+	/*Make new vbox to go in the frame */
+	vbox=gtk_vbox_new(FALSE, 2);
+	gtk_container_add(GTK_CONTAINER(frame), vbox);
+
 	set_win->pos=gtk_check_button_new_with_label(_("Position"));
 	gtk_box_pack_start(GTK_BOX(vbox), set_win->pos, FALSE, FALSE, 2);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(set_win->pos),
@@ -3441,3 +3635,15 @@ err:
 
 	return comment;
 }
+
+UnmountPrompt filer_get_unmount_action(const char *path)
+{
+	return GPOINTER_TO_INT(g_hash_table_lookup(unmount_prompt_actions, path));
+}
+
+void filer_set_unmount_action(const char *path, UnmountPrompt action)
+{
+    g_hash_table_insert(unmount_prompt_actions, g_strdup(path),
+            GINT_TO_POINTER(action));
+    save_learnt_mounts();
+}
