@@ -23,6 +23,9 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
+#ifdef HAVE_LIBINTL_H
+#include <libintl.h>
+#endif
 
 #include "global.h"
 
@@ -40,15 +43,6 @@
 /* Two/five-char country_territory code, or NULL */
 char *current_lang = NULL;
 
-static Option o_translation;
-
-static GtkWidget *i18n_message = NULL;
-
-/* Static Prototypes */
-static void set_trans(const guchar *lang);
-static void trans_changed(void);
-static GList *build_i18n_message(Option *option, xmlNode *node, guchar *label);
-
 /****************************************************************
  *			EXTERNAL INTERFACE			*
  ****************************************************************/
@@ -57,15 +51,37 @@ static GList *build_i18n_message(Option *option, xmlNode *node, guchar *label);
 /* Set things up for internationalisation */
 void i18n_init(void)
 {
+	const char *lang;
+
 	gtk_set_locale();
 
-	option_add_string(&o_translation, "i18n_translation", "From LANG");
-	set_trans(o_translation.value);
-	o_translation.has_changed = FALSE; /* Prevent warning about saving */
+#ifdef HAVE_LIBINTL_H
+	gchar *path = g_strdup_printf("%s/Messages", app_dir);
+	bindtextdomain("ROX-Filer", path);
+	bind_textdomain_codeset("ROX-Filer", "UTF-8");
+	g_free(path);
+#endif
 
-	option_add_notify(trans_changed);
+	/* This is a hang-over from when we did translations ourselves.
+	 * Maybe we can get this info from the gettext library?
+	 */
+	lang = getenv("LANG");
+	if (lang)
+	{
+		const char *end;
 
-	option_register_widget("i18n-message", build_i18n_message);
+		/* Extract the language code from the locale name.
+		 * language[_territory][.codeset][@modifier]
+		 */
+
+		end = strchr(lang, '.');
+		if (!end)
+			end = strchr(lang, '@');
+		if (end)
+			current_lang = g_strndup(lang, end - lang);
+		else
+			current_lang = g_strdup(lang);
+	}
 }
 
 /* These two stolen from dia :-).
@@ -151,99 +167,3 @@ void free_translated_entries(GtkItemFactoryEntry *entries, gint n)
 		g_free(entries[i].path);
 	g_free(entries);
 }
-
-
-/****************************************************************
- *			INTERNAL FUNCTIONS			*
- ****************************************************************/
-
-static void trans_changed(void)
-{
-	if (!o_translation.has_changed)
-		return;
-
-	set_trans(o_translation.value);
-
-	if (i18n_message)
-		gtk_label_set_text(GTK_LABEL(i18n_message), MESSAGE);
-}
-
-/* Load the 'Messages/<name>.gmo' translation.
- * Special values 'None' and 'From LANG' are also allowed.
- */
-static void set_trans(const guchar *lang)
-{
-	guchar	*path;
-	gchar	*lang2;
-
-	g_return_if_fail(lang != NULL);
-
-	rox_clear_translation();
-
-	null_g_free(&current_lang);
-
-	if (strcmp(lang, "None") == 0)
-		return;
-	else if (strcmp(lang, "From LANG") == 0)
-	{
-		const guchar *end;
-
-		lang = getenv("LANG");
-		if (!lang)
-			return;
-		/* Extract the language code from the locale name.
-		 * language[_territory][.codeset][@modifier]
-		 */
-
-		end = strchr(lang, '.');
-		if (!end)
-			end = strchr(lang, '@');
-		if (end)
-			lang2 = g_strndup(lang, end - lang);
-		else
-			lang2 = g_strdup(lang);
-	}
-	else
-		lang2 = g_strdup(lang);
-
-	current_lang = lang2;
-
-	path = g_strdup_printf("%s/Messages/%s.gmo", app_dir, current_lang);
-	if (!file_exists(path) && strchr(current_lang, '_'))
-	{
-		/* Try again without the territory */
-		strcpy(strrchr(path, '_'), ".gmo");
-	}
-
-	if (file_exists(path))
-		rox_add_translations(path);
-	g_free(path);
-}
-
-static GList *build_i18n_message(Option *option, xmlNode *node, guchar *label)
-{
-	GtkWidget *hbox, *image, *align;
-
-	g_return_val_if_fail(option == NULL, NULL);
-	g_return_val_if_fail(label == NULL, NULL);
-	g_return_val_if_fail(i18n_message == NULL, NULL);
-
-	i18n_message = gtk_label_new(MESSAGE);
-	g_signal_connect(i18n_message, "destroy",
-			G_CALLBACK(gtk_widget_destroyed), &i18n_message);
-
-	gtk_misc_set_alignment(GTK_MISC(i18n_message), 0, 0.5);
-	gtk_label_set_justify(GTK_LABEL(i18n_message), GTK_JUSTIFY_LEFT);
-	gtk_label_set_line_wrap(GTK_LABEL(i18n_message), TRUE);
-
-	hbox = gtk_hbox_new(FALSE, 4);
-	image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO,
-			GTK_ICON_SIZE_BUTTON);
-	align = gtk_alignment_new(0, 0, 0, 0);
-
-	gtk_container_add(GTK_CONTAINER(align), image);
-	gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), i18n_message, FALSE, TRUE, 0);
-
-	return g_list_append(NULL, hbox);
-}
