@@ -32,6 +32,13 @@ struct _Panel {
 
 	int		autoscroll_to;	/* Timeout */
 	int		autoscroll_speed; /* 0 => not scrolling */
+	GdkRectangle	geometry;
+	/* Options */
+	int			style;		/* Possible values defined in panel.c */
+	int			width;
+	gboolean	xinerama;
+	int			monitor;
+	gboolean	avoid;
 };
 
 extern Panel *current_panel[PANEL_NUMBER_OF_SIDES];
@@ -48,6 +55,11 @@ gboolean panel_remove_item(PanelSide side, const gchar *path,
 			   const gchar *label);
 void panel_mark_used(GdkRegion *used);
 void panel_update_size(void);
+
+/* Side names here are English for implementation */
 PanelSide panel_name_to_side(gchar *side);
+const char *panel_side_to_name(PanelSide side);
+
+GtkWidget *panel_new_panel_submenu(void);
 
 #endif /* _PANEL_H */
