@@ -18,7 +18,7 @@ extern GdkColor pin_text_fg_col, pin_text_bg_col;
 typedef enum {
 	BACKDROP_NONE,
 	BACKDROP_PROGRAM,
-	BACKDROP_CENTRE, BACKDROP_SCALE, BACKDROP_STRETCH, BACKDROP_TILE
+	BACKDROP_CENTRE, BACKDROP_SCALE, BACKDROP_STRETCH, BACKDROP_TILE, BACKDROP_FIT
 } BackdropStyle;
 
 void pinboard_init(void);
@@ -26,7 +26,8 @@ void pinboard_activate(const gchar *name);
 void pinboard_pin(const gchar *path, const gchar *name, int x, int y,
 		  const gchar *shortcut);
 void pinboard_pin_with_args(const gchar *path, const gchar *name, int x, int y,
-			   const gchar *shortcut, const char *arg, gboolean locked);
+			   const gchar *shortcut, const char *arg, 
+			   gboolean locked, gboolean update);
 gboolean pinboard_remove(const gchar *path, const gchar *name);
 void pinboard_move_icons(void);
 const gchar *pinboard_get_name(void);
