@@ -22,11 +22,12 @@ class Handler(ContentHandler):
 		data = string.strip(self.data)
 		if data:
 			self.trans(data)
+		self.data = ""
 	
 	def trans(self, data):
 		data = string.join(string.split(data, '\n'), '\\n')
 		if data:
-			out.write('_("%s")\n' % data)
+			out.write('_("%s")\n' % data.replace('"', '\\"'))
 
 try:
 	os.chdir("po")
