@@ -87,6 +87,7 @@ static xmlNodePtr rpc_Show(GList *args);
 static xmlNodePtr rpc_Pinboard(GList *args);
 static xmlNodePtr rpc_Panel(GList *args);
 static xmlNodePtr rpc_Run(GList *args);
+static xmlNodePtr rpc_RunURI(GList *args);
 static xmlNodePtr rpc_Copy(GList *args);
 static xmlNodePtr rpc_Move(GList *args);
 static xmlNodePtr rpc_Link(GList *args);
@@ -132,6 +133,7 @@ gboolean remote_init(xmlDocPtr rpc, gboolean new_copy)
 	soap_register("CloseDir", rpc_CloseDir, "Filename", NULL);
 	soap_register("Examine", rpc_Examine, "Filename", NULL);
 	soap_register("Show", rpc_Show, "Directory,Leafname", NULL);
+	soap_register("RunURI", rpc_RunURI, "URI", NULL);
 
 	soap_register("Pinboard", rpc_Pinboard, NULL, "Name");
 	soap_register("Panel", rpc_Panel, NULL, "Side,Name");
@@ -146,7 +148,7 @@ gboolean remote_init(xmlDocPtr rpc, gboolean new_copy)
 
 	soap_register("SetBackdrop", rpc_SetBackdrop, "Filename,Style", NULL);
 	soap_register("SetBackdropApp", rpc_SetBackdropApp, "App", NULL);
-	soap_register("PinboardAdd", rpc_PinboardAdd, "Path,X,Y", "Label,Shortcut,Args,Locked");
+	soap_register("PinboardAdd", rpc_PinboardAdd, "Path", "X,Y,Label,Shortcut,Args,Locked,Update");
 	soap_register("PinboardRemove", rpc_PinboardRemove, "Path", "Label");
 	soap_register("PanelAdd", rpc_PanelAdd, "Side,Path", "Label,After,Shortcut,Args,Locked");
 	soap_register("PanelRemove", rpc_PanelRemove, "Side", "Path,Label");
@@ -311,6 +313,66 @@ out:
 }
 
 
+/* Parse a SOAP reply and extract any fault strings, returning them as
+ * a NULL terminated list of strings (g_strfreev).
+ */
+gchar **extract_soap_errors(xmlDocPtr reply)
+{
+	gchar **errs;
+	GSList *errlist=NULL, *tmp;
+	int i, n;
+	
+	xmlNodePtr root, node;
+
+	if(!reply)
+		return NULL;
+
+	root=xmlDocGetRootElement(reply);
+	if(strcmp(root->name, "Envelope")==0) {
+		node=get_subnode(root, SOAP_ENV_NS, "Body");
+		if(node) {
+			xmlNodePtr sub, fault;
+			for(sub=node->xmlChildrenNode; sub;
+			    sub=sub->next) {
+				if(sub->type != XML_ELEMENT_NODE)
+					continue;
+				if(strcmp(sub->name, "env:Fault")!=0)
+					continue;
+				
+				/*if (sub->ns == NULL)
+				  continue;
+
+				  if(strcmp(sub->ns->href,
+				  SOAP_ENV_NS)!=0)
+				  continue;*/
+
+				fault=get_subnode(sub, NULL,
+						  "faultstring");
+				if(fault) {
+					xmlChar *txt;
+					txt=xmlNodeGetContent(fault);
+					if(txt) {
+						errlist=g_slist_append(errlist,
+								       g_strdup(txt));
+								       
+						xmlFree(txt);
+					}
+				}
+			}
+		}
+	}
+
+	n=g_slist_length(errlist);
+	errs=g_malloc(sizeof(gchar *)*(n+1));
+	for(i=0, tmp=errlist; i<n; i++, tmp=g_slist_next(tmp))
+		errs[i]=tmp->data;
+	errs[n]=NULL;
+
+	g_slist_free(errlist);
+
+	return errs;
+}
+
 /****************************************************************
  *			INTERNAL FUNCTIONS			*
  ****************************************************************/
@@ -737,6 +799,29 @@ static xmlNodePtr rpc_Run(GList *args)
 	return NULL;
 }
 
+static xmlNodePtr rpc_RunURI(GList *args)
+{
+	char	   *uri, *errmsg=NULL;
+	xmlNodePtr  reply=NULL;
+
+	uri = string_value(ARG(0));
+	run_by_uri(uri, &errmsg);
+	g_free(uri);
+	
+	if(errmsg)
+	{
+		reply = xmlNewNode(NULL, "env:Fault");
+		xmlNewNs(reply, SOAP_RPC_NS, "rpc");
+		xmlNewNs(reply, SOAP_ENV_NS, "env");
+		xmlNewTextChild(reply, NULL, "faultcode",
+						"Failed");
+		xmlNewTextChild(reply, NULL, "faultstring", errmsg);
+		g_free(errmsg);
+	}
+
+	return reply;
+}
+
 static xmlNodePtr rpc_CloseDir(GList *args)
 {
 	char	   *path;
@@ -827,23 +912,25 @@ static xmlNodePtr rpc_SetBackdropApp(GList *args)
 	return NULL;
 }
 
-/* args = Path, X, Y, [Label, Shortcut, Args, Locked] */
+/* args = Path, [X, Y, Label, Shortcut, Args, Locked, Update] */
 static xmlNodePtr rpc_PinboardAdd(GList *args)
 {
 	char *path = NULL;
 	gchar *name, *shortcut, *xargs;
-	int x, y, locked;
+	int x, y, locked, update;
 
 	path = string_value(ARG(0));
-	x = int_value(ARG(1), 0);
-	y = int_value(ARG(2), 0);
+	x = int_value(ARG(1), -1);
+	y = int_value(ARG(2), -1);
 	name = string_value(ARG(3));
 	shortcut = string_value(ARG(4));
 	xargs = string_value(ARG(5));
 	locked = bool_value(ARG(6));
+	update = bool_value(ARG(7));
 
 	pinboard_pin_with_args(path, name, x, y, shortcut, xargs, 
-						(locked==-1) ? FALSE : locked);
+						(locked==-1) ? FALSE : locked, 
+						(update==-1) ? FALSE : update);
 
 	g_free(path);
 	g_free(name);
