@@ -564,9 +564,16 @@ void set_blocking(int fd, gboolean blocking)
 char *pretty_time(const time_t *time)
 {
         char time_buf[32];
+	struct tm *tms;
 
-        if (strftime(time_buf, sizeof(time_buf),
-			TIME_FORMAT, localtime(time)) == 0)
+	if (time == NULL)
+		return g_strdup("(null)");
+
+	tms = localtime(time);
+	if (tms == NULL)
+		return g_strdup("(invalid time)");
+
+        if (strftime(time_buf, sizeof(time_buf), TIME_FORMAT, tms) == 0)
 		time_buf[0]= 0;
 
 	return to_utf8(time_buf);
@@ -1717,3 +1724,42 @@ gchar *build_command_with_path(const char *cmd, const char *path)
 
 	return result;
 }
+
+/* Search for a application on $APPDIRPATH
+ * (~/Apps:/usr/local/apps:/usr/apps) and return a copy of the path found.
+ * Returns NULL if not found
+ */
+gchar *find_app(const char *appname)
+{
+  const gchar *path=g_getenv("APPDIRPATH");
+  gchar **search;
+  gchar *app=NULL;
+  int i;
+
+  if(path)
+  {
+	  search=g_strsplit(path, ":", 0);
+  }
+  else
+  {
+	  gchar *tmp;
+	  
+	  tmp=g_strdup_printf("%s/Apps:/usr/local/apps:/usr/apps",
+			       g_get_home_dir());
+	  search=g_strsplit(tmp, ":", 0);
+	  g_free(tmp);
+  }
+
+  for(i=0; search[i]; i++)
+  {
+	  app=g_strconcat(search[i], "/", appname, NULL);
+	  if(access(app, X_OK)==0)
+		  goto out;
+	  g_free(app);
+  }
+  app=NULL;
+
+ out:
+  g_strfreev(search);
+  return app;
+}
