@@ -1,8 +1,6 @@
 /*
- * $Id: usericons.c,v 1.70 2005/07/16 11:40:11 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -52,6 +50,7 @@
 #include "display.h"
 #include "xml.h"
 #include "dropbox.h"
+#include "icon.h"
 
 #define SET_MEDIA 2
 #define SET_TYPE 1
@@ -64,13 +63,11 @@ static GHashTable *glob_icons = NULL; /* Pathname -> Icon pathname */
 static const char *process_globicons_line(gchar *line);
 static gboolean free_globicon(gpointer key, gpointer value, gpointer data);
 static void write_globicons(void);
-static void add_globicon(const gchar *path, const gchar *icon);
 static void drag_icon_dropped(GtkWidget	*drop_box,
 				  const guchar	*path,
 				  GtkWidget	*dialog);
 static gboolean set_icon_for_type(MIME_type *type, const gchar *iconpath,
 				  gboolean just_media);
-static void delete_globicon(const gchar *path);
 static gboolean convert_to_png(const gchar *src, const gchar *dest);
 static void radios_changed(Radios *radios, gpointer data);
 
@@ -166,6 +163,7 @@ static gboolean create_diricon(const guchar *filepath, const guchar *iconpath)
 		return FALSE;
 
 	dir_check_this(filepath);
+	icons_may_update(filepath);
 
 	return TRUE;
 }
@@ -210,18 +208,17 @@ static void dialog_response(GtkWidget *dialog, gint response, gpointer data)
 static void clear_icon(DropBox *drop_box, GObject *dialog)
 {
 	Radios *radios;
+	const guchar *pathname;
+
+	pathname = g_object_get_data(G_OBJECT(dialog), "pathname");
+	g_return_if_fail(pathname != NULL);
 	
 	radios = g_object_get_data(G_OBJECT(dialog), "radios");
 	g_return_if_fail(radios != NULL);
 
 	if (radios_get_value(radios) == SET_PATH)
 	{
-		const guchar *path;
-
-		path = g_object_get_data(G_OBJECT(dialog), "pathname");
-		g_return_if_fail(path != NULL);
-
-		delete_globicon(path);
+		delete_globicon(pathname);
 	}
 	else
 	{
@@ -241,6 +238,11 @@ static void clear_icon(DropBox *drop_box, GObject *dialog)
 			if (unlink(path))
 				delayed_error(_("Can't delete '%s':\n%s"),
 						path, g_strerror(errno));
+			else
+			{
+				dir_check_this(pathname);
+				icons_may_update(pathname);
+			}
 		}
 		g_free(tmp);
 	}
@@ -489,7 +491,7 @@ static const char *process_globicons_line(gchar *line)
  * added to the top of the list (so that it takes precedence over
  * other entries).
  */
-static void add_globicon(const gchar *path, const gchar *icon)
+void add_globicon(const gchar *path, const gchar *icon)
 {
 	g_hash_table_insert(glob_icons, g_strdup(path), g_strdup(icon));
 
@@ -501,7 +503,7 @@ static void add_globicon(const gchar *path, const gchar *icon)
 }
 
 /* Remove the globicon for a certain path */
-static void delete_globicon(const gchar *path)
+void delete_globicon(const gchar *path)
 {
 	gpointer key, value;
 
