@@ -39,6 +39,9 @@
 #include <errno.h>
 #include <limits.h>
 
+/* ROX: */
+#include <glib.h>
+
 #ifndef	FALSE
 #define	FALSE	(0)
 #endif
@@ -654,6 +657,83 @@ _xdg_mime_magic_get_buffer_extents (XdgMimeMagic *mime_magic)
   return mime_magic->max_extent;
 }
 
+static gboolean _rox_buffer_looks_like_text (const void  *data,
+					    const size_t len)
+{
+  gchar *end;
+
+  if (g_utf8_validate (data, len, (const gchar**)&end))
+  {
+    /* g_utf8_validate allows control characters */
+    int i;
+    for (i = 0; i < len; i++)
+    {
+      unsigned char c = ((const guchar *) data)[i];
+      if (c < 32 && c != '\r' && c != '\n' && c != '\t')
+	return FALSE;
+    }
+    return TRUE;
+
+  } else {
+    /* Check whether the string was truncated in the middle of
+     * a valid UTF8 char, or if we really have an invalid
+     * UTF8 string
+     */
+    gint remaining_bytes = len;
+
+    remaining_bytes -= (end-((gchar*)data));
+
+    if (g_utf8_get_char_validated(end, remaining_bytes) == -2)
+      return TRUE;
+#if defined(HAVE_WCTYPE_H) && defined (HAVE_MBRTOWC)
+    else {
+      size_t wlen;
+      wchar_t wc;
+      gchar *src, *end;
+      mbstate_t state;
+
+      src = data;
+      end = data+len;
+                       
+      memset (&state, 0, sizeof (state));
+      while (src < end) {
+	/* Don't allow embedded zeros in textfiles */
+	if (*src == 0)
+	  return FALSE;
+                               
+	wlen = mbrtowc(&wc, src, end - src, &state);
+
+	if (wlen == (size_t)(-1)) {
+	  /* Illegal mb sequence */
+	  return FALSE;
+	}
+                               
+	if (wlen == (size_t)(-2)) {
+	  /* No complete mb char before end
+	   * Probably a cut off char which is ok */
+	  return TRUE;
+	}
+
+	if (wlen == 0) {
+	  /* Don't allow embedded zeros in textfiles */
+	  return FALSE;
+	}
+        
+	if (!iswspace (wc)  && !iswprint(wc)) {
+	  /* Not a printable or whitspace
+	   * Probably not a text file */
+	  return FALSE;
+	}
+	
+	src += wlen;
+      }
+      return TRUE;
+    }
+#endif /* defined(HAVE_WCTYPE_H) && defined (HAVE_MBRTOWC) */
+  }
+  return FALSE;
+}
+
 const char *
 _xdg_mime_magic_lookup_data (XdgMimeMagic *mime_magic,
 			     const void   *data,
@@ -680,7 +760,10 @@ _xdg_mime_magic_lookup_data (XdgMimeMagic *mime_magic,
 	      mime_type = match->mime_type;
 	      priority = match->priority;
 	    }
+	  /* Is this another match at same priority which is not the same
+	   * type again, or a sub-type */
 	  else if (had_match && match->priority == priority && mime_type &&
+		   strcmp(mime_type, match->mime_type)!=0 &&
 		   !_xdg_mime_mime_type_subclass (mime_type, match->mime_type))
 	    /* multiple unrelated patterns with the same priority matched,
 	     * so we can't tell what type this is. */
@@ -708,6 +791,12 @@ _xdg_mime_magic_lookup_data (XdgMimeMagic *mime_magic,
 	}
     }
 
+  if (mime_type == NULL)
+    {
+      if(_rox_buffer_looks_like_text(data, len))
+	 mime_type = XDG_MIME_TYPE_UNKNOWN_TEXT;
+    }
+  
   return mime_type;
 }
 
