@@ -1,8 +1,6 @@
 /*
- * $Id: xtypes.c,v 1.17 2005/08/16 17:59:48 kerofin Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -41,9 +39,7 @@
 #include "xtypes.h"
 #include "options.h"
 
-static Option o_xattr_ignore;
-
-#define XTYPE_ATTR "user.mime_type"
+Option o_xattr_ignore;
 
 #define RETURN_IF_IGNORED(val) if(o_xattr_ignore.int_value) return (val)
 
@@ -64,6 +60,12 @@ void xattr_init(void)
 	void *libc;
 	
 	libc = dlopen("libc.so.6", RTLD_LAZY | RTLD_NOLOAD);
+	if (!libc)
+	{
+		/* Try a different name for uClib support */
+		libc = dlopen("libc.so", RTLD_LAZY | RTLD_NOLOAD);
+	}
+
 	if (!libc)
 		return;	/* Give up on xattr support */
 
@@ -86,7 +88,7 @@ int xattr_supported(const char *path)
 
 	if(path) {
 		errno=0;
-		nent=dyn_getxattr(path, XTYPE_ATTR, buf, sizeof(buf));
+		nent=dyn_getxattr(path, XATTR_MIME_TYPE, buf, sizeof(buf));
 
 		if(nent<0 && errno==ENOTSUP)
 			return FALSE;
@@ -304,7 +306,7 @@ MIME_type *xtype_get(const char *path)
 	gchar *buf;
 	char *nl;
 
-	buf = xattr_get(path, XTYPE_ATTR, NULL);
+	buf = xattr_get(path, XATTR_MIME_TYPE, NULL);
 
 	if(buf)
 	{
@@ -329,7 +331,7 @@ int xtype_set(const char *path, const MIME_type *type)
 	}
 
 	ttext = g_strdup_printf("%s/%s", type->media_type, type->subtype);
-	res = xattr_set(path, XTYPE_ATTR, ttext, -1);
+	res = xattr_set(path, XATTR_MIME_TYPE, ttext, -1);
 	g_free(ttext);
 
 	return res;
